<?php

namespace Illuminate\Foundation\Auth;

use Illuminate\Auth\Events\Registered;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use App\Models\Country;

trait RegistersUsers
{
    use RedirectsUsers;

    /**
     * Show the application registration form.
     *
     * @return \Illuminate\View\View
     */
    public function showRegistrationForm()
    {
        return view('auth.register');
    }

    /**
     * Handle a registration request for the application.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\RedirectResponse|\Illuminate\Http\JsonResponse
     */
     
    public static function calculateZipCodeSimilarity($zipCode1, $zipCode2) {
        return levenshtein($zipCode1, $zipCode2);
    }

    public static function isSimilarZipCode($zipCode1, $zipCode2, $threshold) {
        $similarity = static::calculateZipCodeSimilarity($zipCode1, $zipCode2);
        return $similarity <= $threshold;
    }

    public function register(Request $request)
    {
        $this->validator($request->all())->validate();
        
        $controller = resolve(\App\Http\Controllers\IPQSController::class);
        $email_result = $controller->emailVerification($request->email);

        if(isset($email_result) && $email_result->valid !== true){
            return back()->with('err', 'Please enter a valid email');
        }
        
        $controller = resolve(\App\Http\Controllers\IPQSController::class);
        $result = $controller->proxyDetection($_SERVER['REMOTE_ADDR']);

        $zipCode1 = $result->zip_code;
        $zipCode2 = $request->postal_code;
        $similarityThreshold = 2; // Adjust the threshold based on your desired similarity level

        if (!static::isSimilarZipCode($zipCode1, $zipCode2, $similarityThreshold)) {
            return back()->with('err', 'Your postal code does not match with our record. Please enter a valid postal code to register');
        }

        $country_code = Country::where('id', $request->country)->select('code')->value('code');
        if($result->country_code !== $country_code) {
            return back()->with('err', 'Your selected country does not match with our record. Please select a valid country to register');
        }

        event(new Registered($user = $this->create($request->all())));

        $this->guard()->login($user);

        if ($response = $this->registered($request, $user)) {
            return $response;
        }

        return $request->wantsJson()
                    ? new JsonResponse([], 201)
                    : redirect($this->redirectPath());
    }

    /**
     * Get the guard to be used during registration.
     *
     * @return \Illuminate\Contracts\Auth\StatefulGuard
     */
    protected function guard()
    {
        return Auth::guard();
    }

    /**
     * The user has been registered.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  mixed  $user
     * @return mixed
     */
    protected function registered(Request $request, $user)
    {
        //
    }
}
